==> SyFi-0.2.0.tar.gz: SyFi-0.2.0/syfi/ElementComputations.cpp <==
#include <ElementComputations.h>

void usage(FE& fe) {
  for (int i=1; i<= fe.nbf(); i++) {
    cout <<"fe.N("<<i<<")         =   "<<fe.N(i)<<endl;
    cout <<"grad(fe.N("<<i<<"))   =   "<<grad(fe.N(i))<<endl;
    cout <<"fe.dof("<<i<<")       =   "<<fe.dof(i)<<endl;
  }
}

void usage(FE& v_fe, FE& p_fe) {
  for (int i=1; i<= v_fe.nbf(); i++) {
    cout <<"v_fe.N("<<i<<")         =   "<<v_fe.N(i)<<endl;
    cout <<"grad(v_fe.N("<<i<<"))   =   "<<grad(v_fe.N(i))<<endl;
    cout <<"v_fe.dof("<<i<<")       =   "<<v_fe.dof(i)<<endl;
  }
  for (int i=1; i<= p_fe.nbf(); i++) {
    cout <<"p_fe.N("<<i<<")=   "<<p_fe.N(i)<<endl;
    cout <<"p_fe.dof("<<i<<")= "<<p_fe.dof(i)<<endl;
  }
}



void compute_Poisson_element_matrix(
		FE& fe, 
                Dof& dof, 
		std::map<std::pair<int,int>, GiNaC::ex>& A)
{
  std::pair<int,int> index; 

   // Insert the local degrees of freedom into the global Dof
  for (int i=1; i<= fe.nbf(); i++) {
    dof.insert_dof(1,i,fe.dof(i)); 
  }

  Polygon& domain = fe.getPolygon(); 

  // The term (grad u, grad v)  
  for (int i=1; i<= fe.nbf(); i++) {
    index.first = dof.glob_dof(fe.dof(i));       // fetch the global dof for Ni 
    for (int j=1; j<= fe.nbf(); j++) {
      index.second = dof.glob_dof(fe.dof(j));    // fetch the global dof for Nj 
      GiNaC::ex nabla = inner(grad(fe.N(i)),     // compute the integrand
		              grad(fe.N(j))); 
      GiNaC::ex Aij = domain.integrate(nabla);   // compute the integral  
      A[index] += Aij;                           // add to global matrix 
    }
  }
}



void compute_Stokes_element_matrix(
		FE& v_fe, 
		FE& p_fe,
                Dof& dof, 
		std::map<std::pair<int,int>, GiNaC::ex>& A)
{
  std::pair<int,int> index; 
  std::pair<int,int> index2; 

  // FIXME: need to check that p_fe 
  // contains the same domain
  Polygon& domain = v_fe.getPolygon(); 

   // Insert the local degrees of freedom into the global Dof
  for (int i=1; i<= v_fe.nbf(); i++) {
    dof.insert_dof(1,i,v_fe.dof(i)); 
  }
  for (int i=1; i<= p_fe.nbf(); i++) {
    dof.insert_dof(1,v_fe.nbf()+i,p_fe.dof(i)); 
  }




  // The term (grad u, grad v)  
  for (int i=1; i<= v_fe.nbf(); i++) {
    index.first = dof.glob_dof(v_fe.dof(i));     // fetch the global dof for v_i 
    for (int j=1; j<= v_fe.nbf(); j++) {
      index.second = dof.glob_dof(v_fe.dof(j));  // fetch the global dof for v_j 
      GiNaC::ex nabla = inner(grad(v_fe.N(i)),
		              grad(v_fe.N(j)));  // compute the integrand  
      GiNaC::ex Aij = domain.integrate(nabla);   // compute the integral  
      A[index] += Aij;                           // add to global matrix 
    }
  }



  // The term -(div u, q)  
  for (int i=1; i<= p_fe.nbf(); i++) {
    index.first = dof.glob_dof(p_fe.dof(i));     // fetch the global dof for p_i  
    for (int j=1; j<= v_fe.nbf(); j++) {
      index.second=dof.glob_dof(v_fe.dof(j));    // fetch the global dof for v_j  
      GiNaC::ex divV= -p_fe.N(i)*div(v_fe.N(j));  // compute the integrand 
      GiNaC::ex Aij = domain.integrate(divV);    // compute the integral  
      A[index] += Aij;                           // add to global matrix  

      // Do not need to compute the term (grad(p),v), since the system is 
      // symmetric. We simply set Aji = Aij 
      index2.first = index.second; 
      index2.second = index.first; 
      A[index2] += Aij; 
    }
  }
}



void compute_mixed_Poisson_element_matrix(
		FE& v_fe, 
		FE& p_fe,
                Dof& dof, 
		std::map<std::pair<int,int>, GiNaC::ex>& A)
{
  std::pair<int,int> index; 
  std::pair<int,int> index2; 

  // FIXME: need to check that p_fe 
  // contains the same domain
  Polygon& domain = v_fe.getPolygon(); 

   // Insert the local degrees of freedom into the global Dof
  for (int i=1; i<= v_fe.nbf(); i++) {
    dof.insert_dof(1,i,v_fe.dof(i)); 
  }
  for (int i=1; i<= p_fe.nbf(); i++) {
    dof.insert_dof(1,v_fe.nbf()+i,p_fe.dof(i)); 
  }



  // The term (u,v)  
  for (int i=1; i<= v_fe.nbf(); i++) {
    index.first = dof.glob_dof(v_fe.dof(i));     // fetch the global dof related to i and v   
    for (int j=1; j<= v_fe.nbf(); j++) {
      index.second = dof.glob_dof(v_fe.dof(j));  // fetch the global dof related to j and p 
      GiNaC::ex mass = inner(v_fe.N(i),v_fe.N(j));            // compute the integrand
      GiNaC::ex Aij = domain.integrate(mass);          // compute the integral  
      A[index] += Aij;                           // add to global matrix 
    }
  }

  // The term -(div u, q)  
  for (int i=1; i<= p_fe.nbf(); i++) {
    index.first = dof.glob_dof(p_fe.dof(i));     // fetch the global dof for p_i  
    for (int j=1; j<= v_fe.nbf(); j++) {
      index.second=dof.glob_dof(v_fe.dof(j));    // fetch the global dof for v_j  
      GiNaC::ex divV= -p_fe.N(i)*div(v_fe.N(j));  // compute the integrand 
      GiNaC::ex Aij = domain.integrate(divV);    // compute the integral  
      A[index] += Aij;                           // add to global matrix  

      // Do not need to compute the term (grad(p),v), since the system is 
      // symmetric we simply set Aji = Aij 
      index2.first = index.second; 
      index2.second = index.first; 
      A[index2] += Aij; 
    }
  }
}

==> SyFi-0.3.0.tar.gz: SyFi-0.3.0/syfi/ElementComputations.cpp <==
#include <ElementComputations.h>

void usage(FE& fe) {
  for (int i=0; i< fe.nbf(); i++) {
    cout <<"fe.N("<<i<<")         =   "<<fe.N(i)<<endl;
    cout <<"grad(fe.N("<<i<<"))   =   "<<grad(fe.N(i))<<endl;
    cout <<"fe.dof("<<i<<")       =   "<<fe.dof(i)<<endl;
  }
}

void usage(FE& v_fe, FE& p_fe) {
  for (int i=0; i< v_fe.nbf(); i++) {
    cout <<"v_fe.N("<<i<<")         =   "<<v_fe.N(i)<<endl;
    cout <<"grad(v_fe.N("<<i<<"))   =   "<<grad(v_fe.N(i))<<endl;
    cout <<"v_fe.dof("<<i<<")       =   "<<v_fe.dof(i)<<endl;
  }
  for (int i=0; i< p_fe.nbf(); i++) {
    cout <<"p_fe.N("<<i<<")=   "<<p_fe.N(i)<<endl;
    cout <<"p_fe.dof("<<i<<")= "<<p_fe.dof(i)<<endl;
  }
}



void compute_Poisson_element_matrix(
		FE& fe, 
                Dof& dof, 
		std::map<std::pair<int,int>, GiNaC::ex>& A)
{
  std::pair<int,int> index; 

   // Insert the local degrees of freedom into the global Dof
  for (int i=0; i< fe.nbf(); i++) {
    dof.insert_dof(1,i,fe.dof(i)); 
  }

  Polygon& domain = fe.getPolygon(); 

  // The term (grad u, grad v)  
  for (int i=0; i< fe.nbf(); i++) {
    index.first = dof.glob_dof(fe.dof(i));       // fetch the global dof for Ni 
    for (int j=0; j< fe.nbf(); j++) {
      index.second = dof.glob_dof(fe.dof(j));    // fetch the global dof for Nj 
      GiNaC::ex nabla = inner(grad(fe.N(i)),     // compute the integrand
		              grad(fe.N(j))); 
      GiNaC::ex Aij = domain.integrate(nabla);   // compute the integral  
      A[index] += Aij;                           // add to global matrix 
    }
  }
}



void compute_Stokes_element_matrix(
		FE& v_fe, 
		FE& p_fe,
                Dof& dof, 
		std::map<std::pair<int,int>, GiNaC::ex>& A)
{
  std::pair<int,int> index; 
  std::pair<int,int> index2; 

  // FIXME: need to check that p_fe 
  // contains the same domain
  Polygon& domain = v_fe.getPolygon(); 

   // Insert the local degrees of freedom into the global Dof
  for (int i=0; i< v_fe.nbf(); i++) {
    dof.insert_dof(1,i,v_fe.dof(i)); 
  }
  for (int i=0; i< p_fe.nbf(); i++) {
    dof.insert_dof(1,v_fe.nbf()+i,p_fe.dof(i)); 
  }




  // The term (grad u, grad v)  
  for (int i=0; i< v_fe.nbf(); i++) {
    index.first = dof.glob_dof(v_fe.dof(i));     // fetch the global dof for v_i 
    for (int j=0; j< v_fe.nbf(); j++) {
      index.second = dof.glob_dof(v_fe.dof(j));  // fetch the global dof for v_j 
      GiNaC::ex nabla = inner(grad(v_fe.N(i)),
		              grad(v_fe.N(j)));  // compute the integrand  
      GiNaC::ex Aij = domain.integrate(nabla);   // compute the integral  
      A[index] += Aij;                           // add to global matrix 
    }
  }



  // The term -(div u, q)  
  for (int i=0; i< p_fe.nbf(); i++) {
    index.first = dof.glob_dof(p_fe.dof(i));     // fetch the global dof for p_i  
    for (int j=0; j< v_fe.nbf(); j++) {
      index.second=dof.glob_dof(v_fe.dof(j));    // fetch the global dof for v_j  
      GiNaC::ex divV= -p_fe.N(i)*div(v_fe.N(j));  // compute the integrand 
      GiNaC::ex Aij = domain.integrate(divV);    // compute the integral  
      A[index] += Aij;                           // add to global matrix  

      // Do not need to compute the term (grad(p),v), since the system is 
      // symmetric. We simply set Aji = Aij 
      index2.first = index.second; 
      index2.second = index.first; 
      A[index2] += Aij; 
    }
  }
}



void compute_mixed_Poisson_element_matrix(
		FE& v_fe, 
		FE& p_fe,
                Dof& dof, 
		std::map<std::pair<int,int>, GiNaC::ex>& A)
{
  std::pair<int,int> index; 
  std::pair<int,int> index2; 

  // FIXME: need to check that p_fe 
  // contains the same domain
  Polygon& domain = v_fe.getPolygon(); 

   // Insert the local degrees of freedom into the global Dof
  for (int i=0; i< v_fe.nbf(); i++) {
    dof.insert_dof(1,i,v_fe.dof(i)); 
  }
  for (int i=0; i< p_fe.nbf(); i++) {
    dof.insert_dof(1,v_fe.nbf()+i+1,p_fe.dof(i)); 
  }



  // The term (u,v)  
  for (int i=0; i< v_fe.nbf(); i++) {
    index.first = dof.glob_dof(v_fe.dof(i));     // fetch the global dof related to i and v   
    for (int j=0; j< v_fe.nbf(); j++) {
      index.second = dof.glob_dof(v_fe.dof(j));  // fetch the global dof related to j and p 
      GiNaC::ex mass = inner(v_fe.N(i),v_fe.N(j));            // compute the integrand
      GiNaC::ex Aij = domain.integrate(mass);          // compute the integral  
      A[index] += Aij;                           // add to global matrix 
    }
  }

  // The term -(div u, q)  
  for (int i=0; i< p_fe.nbf(); i++) {
    index.first = dof.glob_dof(p_fe.dof(i));     // fetch the global dof for p_i  
    for (int j=0; j< v_fe.nbf(); j++) {
      index.second=dof.glob_dof(v_fe.dof(j));    // fetch the global dof for v_j  
      GiNaC::ex divV= -p_fe.N(i)*div(v_fe.N(j));  // compute the integrand 
      GiNaC::ex Aij = domain.integrate(divV);    // compute the integral  
      A[index] += Aij;                           // add to global matrix  

      // Do not need to compute the term (grad(p),v), since the system is 
      // symmetric we simply set Aji = Aij 
      index2.first = index.second; 
      index2.second = index.first; 
      A[index2] += Aij; 
    }
  }
}
